"""
"""

import os, sys
import glob

import xml.dom as dom
import xml.sax as sax
import xml.dom.pulldom as pulldom

import maya.OpenMaya as OpenMaya
import maya.cmds as cmds
import maya.mel as mel

import airstream
from airstream.util import odict
from airstream.util import trace

class ImageFormat:
    def __init__( self, name, driver ):
        self.name   = name
        self.driver = driver
        pass

class RenderPass:
    def __init__( self, name ):
        self.name = name
        pass

class PassPreset:
    def __init__( self, name, driver, passName, params ):
        self.name       = name
        self.driver     = driver
        self.passName   = passName
        self.params     = params
        pass

class ResPreset:
    def __init__( self, name, width, height, far, par ):
        self.name               = name
        self.width              = int(width)
        self.height             = int(height)
        self.frameAspectRatio   = float(far)
        self.pixelAspectRatio   = float(par)
        pass

class ShadingRatePreset:
    def __init__( self, name, rate ):
        self.name    = name
        self.rate    = float(rate)
        pass

class PixSamplesPreset:
    def __init__( self, name, width, height ):
        self.name    = name
        self.width   = int(width)
        self.height  = int(height)
        pass

class PixFilterPreset:
    def __init__( self, name, filter, width, height ):
        self.name   = name
        self.filter = filter
        self.width  = int(width)
        self.height = int(height)
        pass

class AirStreamConfig:
    def __init__( self ):
        self.MyImageFormats         = odict()
        self.MyImageFormatsByDriver = odict()
        self.MyRenderPasses         = odict()
        self.MyPassPresets          = odict()
        self.MyResPresets           = odict()
        self.MyShadingRatePresets   = odict()
        self.MyPixSamplesPresets    = odict()
        self.MyPixFilterPresets     = odict()
        self.MyFilePathAttrIgnores        = []
        pass

gAirStreamConfig = None

## Load the configuration
def loadAirStreamConfig():
    global gAirStreamConfig
    gAirStreamConfig = AirStreamConfig()
    try:
        if airstream.kAirStreamHomePath is None:
            raise StandardError( "Envronment variable AIRSTREAM_HOME is not set" )
        configFilePath = os.path.join( airstream.kAirStreamHomePath, "conf", "airstreamConfig.xml" )
        configFile = open( configFilePath )
        events = pulldom.parse( configFile )
        for (event, node) in events:
            # Image Formats
            # -
            if event == pulldom.START_ELEMENT and node.tagName == "format":
                try:
                    events.expandNode( node )
                    fmtName   = str( node.getAttribute( "name" ) )
                    fmtDriver = str( node.getAttribute( "driver" ) )
                    # Insert into the image format dictionary
                    gAirStreamConfig.MyImageFormats[fmtName] = ImageFormat( fmtName, fmtDriver )
                    gAirStreamConfig.MyImageFormatsByDriver[fmtDriver] = ImageFormat( fmtName, fmtDriver )
                except StandardError, msg:
                    raise StandardError( "(ImageFormat): %s" % msg )
            # Render Passes
            elif event == pulldom.START_ELEMENT and node.tagName == "pass":
                try:
                    events.expandNode( node )
                    passName = str( node.getAttribute( "name" ) )
                    # Insert into the render pass dictionary
                    gAirStreamConfig.MyRenderPasses[passName] = RenderPass( passName )
                except StandardError, msg:
                    raise StandardError( "(RenderPass): %s" % msg )
            # Render Pass Presets
            elif event == pulldom.START_ELEMENT and node.tagName == "passpreset":
                try:
                    events.expandNode( node )
                    ppName    = str(node.getAttribute( "name" ))
                    ppDriver  = str(node.getAttribute( "driver" ))
                    rpPass    = str(node.getAttribute( "pass" ))
                    ppParams  = str(node.getAttribute( "params" ))
                    # Insert into the render pass preset dictionary
                    gAirStreamConfig.MyPassPresets[ppName] = PassPreset( ppName, ppDriver, rpPass, ppParams )
                except StandardError, msg:
                    raise StandardError( "(PassPreset): %s" % msg )
            # Resolution Presets
            elif event == pulldom.START_ELEMENT and node.tagName == "respreset":
                try:
                    events.expandNode( node )
                    rpName   = str(node.getAttribute( "name" ))
                    rpWidth  = str(node.getAttribute( "width" ))
                    rpHeight = str(node.getAttribute( "height" ))
                    rpFar    = str(node.getAttribute( "far" ))
                    rpPar    = str(node.getAttribute( "par" ))
                    # Insert into the resolution preset dictionary
                    gAirStreamConfig.MyResPresets[rpName] =  ResPreset( rpName, rpWidth, rpHeight, rpFar, rpPar )
                except StandardError, msg:
                    raise StandardError( "(ResPreset): %s" % msg )
            # Shading Rate Presets
            elif event == pulldom.START_ELEMENT and node.tagName == "shadingRatePreset":
                try:
                    events.expandNode( node )
                    srpName = str(node.getAttribute( "name" ))
                    srpRate = str(node.getAttribute( "rate" ))
                    # Insert into the resolution preset dictionary
                    gAirStreamConfig.MyShadingRatePresets[srpName] =  ShadingRatePreset( srpName, srpRate )
                except StandardError, msg:
                    raise StandardError( "(ShadingRatePreset): %s" % msg )
            # Pixel Samples Preset
            elif event == pulldom.START_ELEMENT and node.tagName == "pixSamplesPreset":
                try:
                    events.expandNode( node )
                    pspName   = str(node.getAttribute( "name" ))
                    pspWidth  = str(node.getAttribute( "width" ))
                    pspHeight = str(node.getAttribute( "height" ))
                    # Insert into the resolution preset dictionary
                    gAirStreamConfig.MyPixSamplesPresets[pspName] =  PixSamplesPreset( pspName, pspWidth, pspHeight )
                except StandardError, msg:
                    raise StandardError( "(PixSamplesPreset): %s" % msg )
            # Pixel Filter Preset
            elif event == pulldom.START_ELEMENT and node.tagName == "pixFilterPreset":
                try:
                    events.expandNode( node )
                    pfpName   = str(node.getAttribute( "name" ))
                    pfpFilter = str(node.getAttribute( "filter" ))
                    pfpWidth  = str(node.getAttribute( "width" ))
                    pfpHeight = str(node.getAttribute( "height" ))
                    # Insert into the resolution preset dictionary
                    gAirStreamConfig.MyPixFilterPresets[pfpName] =  PixFilterPreset( pfpName, pfpFilter, pfpWidth, pfpHeight )
                except StandardError, msg:
                    raise StandardError( "(PixSamplesPreset): %s" % msg )
            # File Path Attribute Name
            elif event == pulldom.START_ELEMENT and node.tagName == "filePathAttrIgnore":
                try:
                    events.expandNode( node )
                    fpaiName = str(node.getAttribute( "name" ))
                    # Insert into the File Path Attribute Name array
                    gAirStreamConfig.MyFilePathAttrIgnores.append( fpaiName )
                except StandardError, msg:
                    raise StandardError( "(PixSamplesPreset): %s" % msg )
        print( "[PYDBG] : Loaded config in Python layer from %s" % configFilePath )
    except StandardError, msg:
        OpenMaya.MGlobal.displayError( "Error loading %s: %s" % (configFilePath, msg) )
    pass

## Initializes the default values for airstreamGlobals right after it's created
#
def initGlobalsDefaults( node ):
    print( "[PYDBG] : initGlobalsDefaults" )
    workspaceRoot = cmds.workspace( q=True, rd=True )

    airhome = os.path.expandvars( "$AIRHOME" )
    cmds.setAttr( "%s.airHome" % node, airhome, type="string" )
    cmds.setAttr( "%s.ribExportType " % node, 0 )

    # Turn this on by default
    cmds.setAttr( "%s.inheritAIRStreamProjDir" % node, True )

    airstreamRoot = os.path.join( workspaceRoot, "AIRStream" ).replace( "\\", "/" )
    cmds.setAttr( "%s.airstreamProjectDir" % node, airstreamRoot, type="string" )
    setProjectSubDirs( node, airstreamRoot )

    # Don't append the paths from above into here. It will be difficult to
    # replace them later. Instead we'll add them during export time.
    archiveSearchPath    = "&"
    displaySearchPath    = "&"
    produceralSearchPath = "&"
    resourceSearchPath   = "&"
    shaderSearchPath     = "&"
    textureSearchPath    = "&"
    cmds.setAttr( "%s.archiveSearchPath"    % node, archiveSearchPath,    type="string" )
    cmds.setAttr( "%s.displaySearchPath"    % node, displaySearchPath,    type="string" )
    cmds.setAttr( "%s.produceralSearchPath" % node, produceralSearchPath, type="string" )
    cmds.setAttr( "%s.resourceSearchPath"   % node, resourceSearchPath,   type="string" )
    cmds.setAttr( "%s.shaderSearchPath"     % node, shaderSearchPath,     type="string" )
    cmds.setAttr( "%s.textureSearchPath"    % node, textureSearchPath,    type="string" )
    pass

##
#
def setProjectSubDirs( node, airstreamRoot ):
    print( "[PYDBG] : setProjectSubDirs node=%s, airstreamRoot=%s" % (node,airstreamRoot) )

    ribDir        = os.path.join( airstreamRoot, "rib"      ).replace( "\\", "/" )
    archiveDir    = os.path.join( airstreamRoot, "archive"  ).replace( "\\", "/" )
    imagesDir     = os.path.join( airstreamRoot, "images"   ).replace( "\\", "/" )
    shadersDir    = os.path.join( airstreamRoot, "shaders"  ).replace( "\\", "/" )
    texturesDir   = os.path.join( airstreamRoot, "textures" ).replace( "\\", "/" )
    tempDir       = os.path.join( airstreamRoot, "temp"     ).replace( "\\", "/" )

    cmds.setAttr( "%s.ribDir"              % node, ribDir,        type="string" )
    cmds.setAttr( "%s.archiveDir"          % node, archiveDir,    type="string" )
    cmds.setAttr( "%s.imagesDir"           % node, imagesDir,     type="string" )
    cmds.setAttr( "%s.shadersDir"          % node, shadersDir,   type="string" )
    cmds.setAttr( "%s.texturesDir"         % node, texturesDir,    type="string" )
    cmds.setAttr( "%s.tempDir"             % node, tempDir,       type="string" )
    pass

## Initializes the default values for airstreamPrimaryRenderPass right after it's created
#
def initPrimaryRenderPassDefaults( node ):
    print( "[PYDBG] : initPrimaryRenderPassDefaults" )
    pass

## Creates the directories from the AIR Stream globals
#
def createAirStreamDirs( node ):
    print( "[PYDBG] : createAirStreamDirs node=%s" % node )

    airstreamRoot = cmds.getAttr( "%s.airstreamProjectDir" % node )
    ribDir        = cmds.getAttr( "%s.ribDir"      % node )
    archiveDir    = cmds.getAttr( "%s.archiveDir"  % node )
    imagesDir     = cmds.getAttr( "%s.imagesDir"   % node )
    texturesDir   = cmds.getAttr( "%s.shadersDir"  % node )
    shadersDir    = cmds.getAttr( "%s.texturesDir" % node )
    tempDir       = cmds.getAttr( "%s.tempDir"     % node )

    dirsToCreate = [ airstreamRoot, ribDir, imagesDir, shadersDir, texturesDir, tempDir ]
    for dtc in dirsToCreate:
        if not os.path.exists( dtc ):
            try:
                os.makedirs( dtc )
                trace( "as_info > Created directory: %s" % dtc )
            except:
                trace( "as_error> Unable to create director: %s" % dtc )
    pass

## Returns a list of transform node names for renderable cameras
#
def getRenderableCameras():
    result = []
    cameras = cmds.ls( cameras=True )
    for cam in cameras:
        if cmds.getAttr( "%s.renderable" % cam ):
            xform = cmds.listRelatives( cam, path=True, parent=True )
            result.append( str( xform[0] ) )
    return result
    pass

## Returns a tuple of transform node names for renderable, nonrenderables cameras
#
def getCameras():
    renderable = []
    nonRenderable = []
    cameras = cmds.ls( cameras=True )
    for cam in cameras:
        if cmds.getAttr( "%s.renderable" % cam ):
            xform = cmds.listRelatives( cam, path=True, parent=True )
            renderable.append( str( xform[0] ) )
        else:
            xform = cmds.listRelatives( cam, path=True, parent=True )
            nonRenderable.append( str( xform[0] ) )
    return (renderable, nonRenderable)
    pass


## Updates the renderable camera list in the AIRStream Render Globals
#
def updateRenderableCameras( ctrl ):
    print( "[PYDBG] : updateRenderableCameras ctrl=%s" % ctrl )

    (renderable,nonRenderable) = getCameras()

    # Save the current parent
    curParent = cmds.setParent( query=True )

    ## According to the Maya Software MEL code, it's faster
    #  if the control is unmanaged when it's being edited.
    cmds.columnLayout( ctrl, edit=True, visible=False )

    ## Remove contents of the layout first.
    children = cmds.columnLayout( ctrl, query=True, childArray=True )
    if children is not None:
        for child in children:
            cmds.deleteUI( child )

    # Set the parent control
    cmds.setParent( ctrl )

    cmds.setUITemplate( "attributeEditorTemplate", pushTemplate=True )

    # Build control groups for all the renderable cameras
    for cam in renderable:
        cmds.rowLayout( parent=ctrl, nc=2, cw2=[365,30], cl2=["left", "right"] )

        ## The 'data' field in the Renderable Camera drop down the following mapping:
        #  0 - divider
        #  1 - renderable camera
        #  2 - non-renderable camera
        #  3 - Add Renderable Camera
        #
        optMenuCtrl = cmds.optionMenuGrp( label="Renderable Camera", cw=[1,140] )
        cmds.menuItem( label=cam, data=1 )
        if len(nonRenderable) > 0:
            cmds.menuItem( divider=True, data=0 )
            for nrCam in nonRenderable:
                cmds.menuItem( label=nrCam, data=2 )
            cmds.menuItem( divider=True )
            cmds.menuItem( label="Add Renderable Camera", data=3 )
            cmds.optionMenuGrp( optMenuCtrl,
                edit=True,
                changeCommand="airstream.globals.onCameraChange('%s', '%s', '%s')" % (optMenuCtrl,ctrl,cam) )

        if len(renderable) > 1:
            cmds.iconTextButton( style="iconOnly",
                image="removeRenderable.xpm",
                annotation="Set camera to non-renderable",
                width=20, height=20,
                command="airstream.globals.setCameraNonRenderable('%s', '%s')" % (cam,ctrl) )
        else:
            cmds.text( label="" )

        cmds.setParent( ".." )

        # Add a separator not at the last camera
        if cam is not renderable[-1:][0]:
            cmds.separator( parent=ctrl, width=425 )

    cmds.setUITemplate( popTemplate=True )

    # Restore the current parent
    cmds.setParent( curParent )

    # Set the control back to managed
    cmds.columnLayout( ctrl, edit=True, visible=True )
    pass

def setCameraRenderable( cam, ctrl, refresh=True ):
    print( "[PYDBG] : setCameraRenderable cam=%s" % cam )
    cmds.setAttr( "%s.renderable" % cam, True )
    if refresh:
        cmds.evalDeferred( "airstream.globals.updateRenderableCameras('%s')" % ctrl )
    pass

def setCameraNonRenderable( cam, ctrl, refresh=True ):
    print( "[PYDBG] : setCameraNonRenderable cam=%s" % cam )
    cmds.setAttr( "%s.renderable" % cam, False )
    if refresh:
        cmds.evalDeferred( "airstream.globals.updateRenderableCameras('%s')" % ctrl )
    pass

def toggleFirstNonRenderableCamera( ctrl ):
    (renderable,nonRenderable) = getCameras()
    if len(nonRenderable) > 0:
        cam = nonRenderable[0]
        setCameraRenderable( cam, ctrl )
    pass

def onCameraChange( menu, ctrl, curCam ):
    print( "[PYDBG] : onCameraChange menu=%s, ctrl=%s, curCam=%s" % (menu,ctrl,curCam) )
    sel = cmds.optionMenuGrp( menu, query=True, select=True ) - 1
    items = cmds.optionMenuGrp( menu, query=True, itemListShort=True )
    data = cmds.menuItem( items[sel], query=True, data=True )
    if 2 == data:
        cam = cmds.optionMenuGrp( menu, query=True, value=True )
        setCameraNonRenderable( curCam, ctrl, False )
        setCameraRenderable( cam, ctrl, True )
    elif 3 == data:
        toggleFirstNonRenderableCamera( ctrl )
    pass

##
# @param globalsNode
# @param resPresetCtrl
#
def onResPresetChange( globalsNode, resPresetCtrl ):
    print( "[PYDBG] : onResPresetChange globalsNode=%s, ctrl=%s" % (globalsNode, resPresetCtrl) )
    try:
        global gAirStreamConfig
        rpName = str( cmds.optionMenuGrp( resPresetCtrl, query=True, value=True ) ).replace(":", "|").replace("\"", "^")
        if rpName not in gAirStreamConfig.MyResPresets.keys():
            raise StandardError( "% not found in Resolution Presets" )
        rp = gAirStreamConfig.MyResPresets[rpName]
        # Get the control names
        ctrlResX        = mel.eval('uiRes("m_airstreamGlobalsUI.kCtrlResolutionX")')
        ctrlResY        = mel.eval('uiRes("m_airstreamGlobalsUI.kCtrlResolutionY")')
        ctrlMaintAspect = mel.eval('uiRes("m_airstreamGlobalsUI.kCtrlMaintainAspect")')
        ctrlFar         = mel.eval('uiRes("m_airstreamGlobalsUI.kCtrlFrameAspectRatio")')
        ctrlPar         = mel.eval('uiRes("m_airstreamGlobalsUI.kCtrlPixelAspectRatio")')
        # Update the controls
        cmds.intFieldGrp( ctrlResX, edit=True, value1=rp.width )
        cmds.intFieldGrp( ctrlResY, edit=True, value1=rp.height )
        cmds.optionMenuGrp( ctrlMaintAspect, edit=True, sl=1 )
        cmds.floatFieldGrp( ctrlFar, edit=True, enable=True, value1=rp.frameAspectRatio )
        cmds.floatFieldGrp( ctrlPar, edit=True, enable=True, value1=rp.pixelAspectRatio )
        # Update the attribues
        cmds.setAttr( "%s.resolutionX" % globalsNode, rp.width )
        cmds.setAttr( "%s.resolutionY" % globalsNode, rp.height )
        cmds.setAttr( "%s.maintainAspect" % globalsNode, 0 )
        cmds.setAttr( "%s.frameAspectRatio" % globalsNode, rp.frameAspectRatio )
        cmds.setAttr( "%s.pixelAspectRatio" % globalsNode, rp.pixelAspectRatio )

        # We need to update these fields so the camera's resolution
        # shows up correctly.
        cmds.setAttr( "defaultResolution.width", rp.width )
        cmds.setAttr( "defaultResolution.height", rp.height )
        cmds.setAttr( "defaultResolution.deviceAspectRatio", rp.frameAspectRatio )
        cmds.setAttr( "defaultResolution.pixelAspect", rp.pixelAspectRatio )
        cmds.setAttr( "defaultResolution.aspectLock", 0 )
    except StandardError, msg:
        OpenMaya.MGlobal.displayError( "(AIR Stream Globals): %s " % msg )
    pass


##
# @param renderPassNode
# @param passPresetCtrl
#
def fillInRenderPassPreset( renderPassNode, passPresetCtrl ):
    print( "[PYDBG] : fillInRenderPreset renderPassNode=%s" % (renderPassNode) )
    try:
        global gAirStreamConfig
        sel = cmds.optionMenuGrp( passPresetCtrl, query=True, select=True ) - 1
        items = cmds.optionMenuGrp( passPresetCtrl, query=True, itemListShort=True )
        data = cmds.menuItem( items[sel], query=True, data=True )
        if 3 == data:
            ppName = str( cmds.optionMenuGrp( passPresetCtrl, query=True, value=True ) )
            if ppName not in gAirStreamConfig.MyPassPresets.keys():
                raise StandardError( "% not found in Render Pass Presets" )
            pp = gAirStreamConfig.MyPassPresets[ppName]
            # Image Format
            try:
                # We don't need to reindex this even though optionMenuGrp is indexed starts at 1
                # and Python array indexes at 0. The reason for this because we have "[ Custom ]".
                idx = gAirStreamConfig.MyImageFormatsByDriver.keys().index( pp.driver )
                cmds.setAttr( "%s.imageFormat" % renderPassNode, idx )
            except StandardError, msg:
                raise StandardError( "Error setting image format: %s (%s)" % (msg, pp.driver) )
            # Render Pass
            try:
                # We don't need to reindex this even though optionMenuGrp is indexed starts at 1
                # and Python array indexes at 0. The reason for this because we have "[ Custom ]".
                idx = gAirStreamConfig.MyRenderPasses.keys().index( pp.passName )
                cmds.setAttr( "%s.imageMode" % renderPassNode, idx )
            except StandardError, msg:
                raise StandardError( "Error setting image mode (render pass): %s (%s)" % (msg, pp.passName) )
            # params
            cmds.setAttr( "%s.params" % renderPassNode, pp.params )
    except StandardError, msg:
        OpenMaya.MGlobal.displayError( "(AIR Stream Globals): %s " % msg )
    pass

##
# @param node
#
# Note: Don't create any directories in this routine. It becomes a bit
#       difficult to manuage with all the attributes and textField controls.
#       Instead, we'll have the directories get created the first time
#       they're needed.
#
def doProjectDirChange( node ):
    print( "[PYDBG] : doProjectDirChange node=%s" % (node) )
    inherit = cmds.getAttr( "%s.inheritAIRStreamProjDir" % node )
    if inherit:
        airstreamRoot = cmds.getAttr( "%s.airstreamProjectDir" % node )
        if airstreamRoot is not None:
            setProjectSubDirs( node, airstreamRoot )
    pass

## createShadingRatePresets
# @param node
# @param ctrlName
# @param ctrlLabel
#
def createShadingRatePresets( node, ctrlName, ctrlLabel ):
    try:
        global gAirStreamConfig
        changeCmd = "airstream.globals.onShadingRatePresetChanged('%s', '%s')" % (node, ctrlName)
        cmds.optionMenuGrp( ctrlName, label=ctrlLabel, cc=changeCmd )
        for val in gAirStreamConfig.MyShadingRatePresets.keys():
            cmds.menuItem( gAirStreamConfig.MyShadingRatePresets[val].name )
        # Set the index of the drop down. We might get an index not
        # found exception, if so, we just ignore it.
        try:
            shadingRate = str(cmds.getAttr( "%s.shadingRate" % node ))
            idx = gAirStreamConfig.MyShadingRatePresets.keys().index( shadingRate ) + 1
            cmds.optionMenuGrp( ctrlName, edit=True, sl=idx )
        except StandardError, msg:
            pass
    except StandardError, msg:
        OpenMaya.MGlobal.displayError( "(AIR Stream Globals): %s " % msg )
    pass

## onShadingRatePresetChanged
# @param node
# @param ctrlName
#
def onShadingRatePresetChanged( node, ctrlName ):
    try:
        global gAirStreamConfig
        srpName = str( cmds.optionMenuGrp( ctrlName, query=True, value=True ) )
        srpRate = gAirStreamConfig.MyShadingRatePresets[srpName].rate
        if( srpRate > 0 ):
            cmds.setAttr( "%s.shadingRate" % node, srpRate )
    except ValueError, msg:
        OpenMaya.MGlobal.displayError( "(AIR Stream Globals): %s " % msg )
    pass

## createPixSamplesPresets
# @param node
# @param ctrlName
# @param ctrlLabel
#
def createPixSamplesPresets( node, ctrlName, ctrlLabel ):
    try:
        global gAirStreamConfig
        changeCmd = "airstream.globals.onPixSamplesPresetChanged('%s', '%s')" % (node, ctrlName)
        cmds.optionMenuGrp( ctrlName, label=ctrlLabel, cc=changeCmd )
        for val in gAirStreamConfig.MyPixSamplesPresets.keys():
            cmds.menuItem( gAirStreamConfig.MyPixSamplesPresets[val].name )
        # Set the index of the drop down. We might get an index not
        # found exception, if so, we just ignore it.
        try:
            pspWidth  = int(cmds.getAttr( "%s.pixelSamplesX" % node ))
            pspHeight = int(cmds.getAttr( "%s.pixelSamplesY" % node ))
            pspName = "%dx%d" % (pspWidth, pspHeight)
            idx = gAirStreamConfig.MyPixSamplesPresets.keys().index( pspName ) + 1
            cmds.optionMenuGrp( ctrlName, edit=True, sl=idx )
        except ValueError, msg:
            pass
    except StandardError, msg:
        OpenMaya.MGlobal.displayError( "(AIR Stream Globals): %s " % msg )
    pass

## onPixSamplesPresetChanged
# @param node
# @param ctrlName
#
def onPixSamplesPresetChanged( node, ctrlName ):
    try:
        global gAirStreamConfig
        pspName   = str( cmds.optionMenuGrp( ctrlName, query=True, value=True ) )
        pspWidth  = gAirStreamConfig.MyPixSamplesPresets[pspName].width
        pspHeight = gAirStreamConfig.MyPixSamplesPresets[pspName].height
        if pspWidth > 0 and pspHeight > 0:
            cmds.setAttr( "%s.pixelSamplesX" % node, pspWidth )
            cmds.setAttr( "%s.pixelSamplesY" % node, pspHeight )
    except StandardError, msg:
        OpenMaya.MGlobal.displayError( "(AIR Stream Globals): %s " % msg )
    pass

## createPixFilterPresets
# @param node
# @param ctrlName
# @param ctrlLabel
#
def createPixFilterPresets( node, ctrlName, ctrlLabel ):
    try:
        global gAirStreamConfig
        changeCmd = "airstream.globals.onPixFilterPresetChanged('%s', '%s')" % (node, ctrlName)
        cmds.optionMenuGrp( ctrlName, label=ctrlLabel, cc=changeCmd )
        for val in gAirStreamConfig.MyPixFilterPresets.keys():
            cmds.menuItem( gAirStreamConfig.MyPixFilterPresets[val].name )
        # Set the index of the drop down. We might get an index not
        # found exception, if so, we just ignore it.
        try:
            pfpFilter = int(cmds.getAttr( "%s.pixelFilter" % node, asString=True ))
            pfpWidth  = int(cmds.getAttr( "%s.pixelFilterSizeX" % node ))
            pfpHeight = int(cmds.getAttr( "%s.pixelFilterSizeY" % node ))
            pfpName = "%s %dx%d" % (pfpFilter, pfpWidth, pfpHeight)
            airstream.pydbg( "pfpName=%s" % pfpName )
            idx = gAirStreamConfig.MyPixFilterPresets.keys().index( pfpName ) + 1
            cmds.optionMenuGrp( ctrlName, edit=True, sl=idx )
        except ValueError, msg:
            pass
    except StandardError, msg:
        OpenMaya.MGlobal.displayError( "(AIR Stream Globals): %s " % msg )
    pass

## onPixFilterPresetChanged
# @param node
# @param ctrlName
#
def onPixFilterPresetChanged( node, ctrlName ):
    try:
        global gAirStreamConfig
        pfpName   = str( cmds.optionMenuGrp( ctrlName, query=True, value=True ) )
        pfpFilter = gAirStreamConfig.MyPixFilterPresets[pfpName].filter
        pfpWidth  = gAirStreamConfig.MyPixFilterPresets[pfpName].width
        pfpHeight = gAirStreamConfig.MyPixFilterPresets[pfpName].height
        if pfpFilter is not "custom" and pfpWidth > 0 and pfpHeight > 0:
            enums = cmds.attributeQuery( "pixelFilter", node=node, listEnum=True )[0].split(":")
            idx = enums.index( pfpFilter )
            cmds.setAttr( "%s.pixelFilter" % node, idx )
            cmds.setAttr( "%s.pixelFilterSizeX" % node, pfpWidth )
            cmds.setAttr( "%s.pixelFilterSizeY" % node, pfpHeight )
    except StandardError, msg:
        OpenMaya.MGlobal.displayError( "(AIR Stream Globals): %s " % msg )
    pass

## onPixFilterPresetChanged - Returns 'True' if attrName is an attribute
# that should be treated as a file path.
#
# @param attrName
#
def isFilePathAttrIgnore( attrName ):
    global gAirStreamConfig
    uAttrName = attrName.upper()
    for ignoreToken in gAirStreamConfig.MyFilePathAttrIgnores:
        ignoreToken = ignoreToken.upper()
        print( uAttrName, ignoreToken )
        if -1 != uAttrName.find( ignoreToken ):
            return True
    return False
    pass
