"""
AIRStream Python Shader Utility Module
"""

import os, sys
import glob
import re

import xml.dom as dom
import xml.sax as sax
import xml.dom.pulldom as pulldom

import maya.OpenMaya as OpenMaya
import maya.cmds as cmds
import maya.mel as mel

import airstream

def fetchShaderXml(shaderNodeName):
    xml = cmds.getAttr( "%s.shaderParams" % shaderNodeName )
    if cmds.about(linux=True):
        f=open('/tmp/airstreamsltemp.txt','w')
        f.write(xml)
        f.close()
        f=open('/tmp/airstreamsltemp.txt','r')
        xml=f.read()
        f.close()
    return xml
    pass

## Parse @a slbPath and returns the filename portion of the path.
# @param slbPath
#
def shaderNameFromSlbPath( slbPath ):
    #print( "[PYDBG] : shaderNameFromSlbPath slbPath=%s" % slbPath )

    return os.path.splitext( os.path.basename( slbPath ) )[0]
    pass

## Parse through the XML in attribute @shaderParams and returns shader name
# @params shaderNodeName
#
def shaderNameFromXml( shaderNodeName ):
    result = None
    # Parse through the xml and find the name of the shader
    xml = fetchShaderXml(shaderNodeName)
    #cmds.getAttr( "%s.shaderParams" % shaderNodeName )
    if xml is None:
        OpenMaya.MGlobal.displayError( "Shader parameter data missing in %s" % shaderNodeName )
        return result
    events = pulldom.parseString( xml )
    for (event, node) in events:
        if event == pulldom.START_ELEMENT and node.tagName == "shader":
            events.expandNode( node )
            result = str(node.getAttribute( "name" ))
            break;
    # Hand over the result
    return result
    pass

## Looks through the paths define in airstreamGlobals.shaderSearchPath to find the shader.
# @param shaderName
#
def findSlbPathFromName( shaderName ):
    #print( "[PYDBG] : findShaderPathFromName shaderName=%s" % shaderName )

    result = None
    globalsNode = mel.eval( "$tmp=$gAIRStreamGlobalsNodeName" )
    searchPathStr = cmds.getAttr( "%s.shaderSearchPath" % globalsNode )
    if searchPathStr is not None:
        # Do some checks on the name to make sure it's compliant
        shaderName = re.sub( "(?i)\\.slb$", ".slb", shaderName )
        if not shaderName.endswith( ".slb" ):
            shaderName += ".slb"
        # Build an array of paths to look through
        searchPathStr = searchPathStr.replace( "&", os.path.join( os.environ["AIRHOME"], "shaders" ) )
        searchPathStr = re.sub( ":[\\\\/]", "|", searchPathStr )
        searchPathStr = searchPathStr.replace( "\\", "/" )
        paths = [ p.replace("|", ":/") for p in searchPathStr.split( ":" ) ]
        for path in paths:
            slbPath = os.path.join( path, shaderName ).replace( "\\", "/" )
            if os.path.exists( slbPath ):
                result = slbPath
                break
    return result
    pass

## Handles the name change from the UI in the shader AETemplate
# @param newShaderName New shader name from the UI textField
# @param ctrl UI textField control
# @param shaderNodeName Shader node
# @param shaderType Shader Type
#
def handleShaderNameChange( newShaderName, ctrl, shaderNodeName, shaderType ):
    slbPath = findSlbPathFromName( newShaderName )
    if slbPath is None:
        OpenMaya.MGlobal.displayError( "Unable to find .slb file for shader: %s - reverting to previous shader" % newShaderName )
        shaderName = shaderNameFromXml( shaderNodeName )
        cmds.textField( ctrl, edit=True, text = shaderName )
        return
    shaderName   = shaderNameFromXml( shaderNodeName )
    shaderPath   = cmds.getAttr( "%s.shaderPath" % shaderNodeName )
    shaderParams = fetchShaderXml( shaderNodename )
    #cmds.getAttr( "%s.shaderParams" % shaderNodeName )
    # Set the shader path before loading it
    cmds.setAttr( "%s.shaderPath" % shaderNodeName, slbPath, type="string" )
    try:
        if loadShaderParamXML( shaderNodeName, shaderType ):
            # Update the shadername so the name appears correctly
            shaderName = shaderNameFromXml( shaderNodeName )
            cmds.setAttr( "%s.shaderName" % shaderNodeName, shaderName, type="string" )
            # This will creae all the controls for the shader param attributes
            createParamAttributes( shaderNodeName )
        else:
            raise RuntimeError( "Shader param load failed" )
    except:
        OpenMaya.MGlobal.displayError( "Unable to load shader parameters for: %s - reverting to previous shader" % newShaderName )
        cmds.setAttr( "%s.shaderName" % shaderNodeName, shaderName, type="string" )
        cmds.setAttr( "%s.shaderPath" % shaderNodeName, shaderPath, type="string" )
        cmds.setAttr( "%s.shaderParams" % shaderNodeName, shaderParams, type="string" )
        cmds.textField( ctrl, edit=True, text = shaderName )
    pass

## Returns true if the @b shaderPath attribute in the node is not empty.
# @param shaderNodeName
#
def hasShaderPath( shaderNodeName ):
    #print( "[PYDBG] : hasShaderPath shaderNodeName=%s" % shaderNodeName )

    shaderPath = cmds.getAttr( "%s.shaderPath" % shaderNodeName )
    if shaderPath is None:
        return False
    return True
    pass

##
# @param shaderNodeName
#
def setShaderPathFromNameAttr( shaderNodeName ):
    #print( "[PYDBG] : setShaderPathFromNameAttr shaderNodeName=%s" % shaderNodeName )

    shaderName = cmds.getAttr( "%s.shaderName" % shaderNodeName )
    shaderPath = findSlbPathFromName( shaderName )
    if shaderPath is None:
        OpenMaya.MGlobal.displayError( "Unable to locate shader .slb for %s"  % shaderName )
        return False
    attr = "%s.shaderPath" % (shaderNodeName)
    cmds.setAttr( attr, shaderPath, type="string" )
    pass

##
# @param shaderNodeName
#
# @return True - if @b shaderParams attribute has XML, False - if @b shaderParams attribute is empty
#
def hasShaderParamXML( shaderNodeName ):
    #print( "[PYDBG] : hasShaderParamXML shaderNodeName=%s" % shaderNodeName )

    attr = "%s.shaderParams" % shaderNodeName
    xml = cmds.getAttr( attr )
    if xml is None:
        return False
    return True
    pass

## Loads shader parameter data into the @b shaderParams attribute in XML format.
# @param shaderNodeName
#
# This will remove any existing shader attributes.
#
def loadShaderParamXML( shaderNodeName, shaderType ):
    #print( "[PYDBG] : loadShaderParamXML shaderNodeName=%s" % shaderNodeName )

    shaderPath = cmds.getAttr( "%s.shaderPath" % shaderNodeName )
    if shaderPath is None:
        OpenMaya.MGlobal.displayError( "File system path to shader not specified" )
        return False
    cmd = 'airstream -loadShader %s "%s" "%s"' % (shaderNodeName, shaderType, cmds.encodeString( shaderPath ) )
    #print( "[PYDBG] : %s" % cmd )
    mel.eval( cmd )
    shaderParams = cmds.getAttr( "%s.shaderParams" % shaderNodeName )
    if shaderParams is None:
        OpenMaya.MGlobal.displayError( "Failed to load shader parameter information" )
        return False
    return True
    pass

##
# @param camelCaseStr
#
def spaceOutCamelCase( camelCaseStr ):
    pattern = re.compile('([A-Z][A-Z][a-z])|([a-z][A-Z])')
    return pattern.sub(lambda m: m.group()[:1] + " " + m.group()[1:], camelCaseStr)
    pass

##
# @param shaderNodeName
# @param parentCtrl
#
def createParamControls( shaderNodeName, parentCtrl ):
    #print( "[PYDBG] : Creating controls for %s, container is %s" % (shaderNodeName, parentCtrl) )

    ## Check the shader name first, this is so the correct error type is returned to the user
    shaderName = cmds.getAttr( "%s.shaderName" % shaderNodeName )
    if shaderName is None:
        OpenMaya.MGlobal.displayError( "No shader specified" )
        return
    ## Now check the XML data
    #xml = cmds.getAttr( "%s.shaderParams" % shaderNodeName, asString=True )
    xml = None
    mayaVersion = cmds.about( v=True ).split(" ")[0]
    fVer = float(mayaVersion)
    if fVer < 2008 and cmds.about( linux=True ):
        xml = mel.eval( "getAttr %s.shaderParams" % shaderNodeName )
    else:
        xml = fetchShaderXml(shaderNodeName)
        #cmds.getAttr( "%s.shaderParams" % shaderNodeName, asString=True )
    if xml is None:
        OpenMaya.MGlobal.displayError( "Shader parameter data missing in %s" % shaderNodeName )
        return
    if "" == xml:
        OpenMaya.MGlobal.displayError( "Shader parameter is empty in %s" % shaderNodeName )
        return
    cmds.setParent( parentCtrl )
    events = pulldom.parseString( xml )
    for (event, node) in events:
        if event == pulldom.START_ELEMENT and node.tagName == "param":
            events.expandNode( node )
            paramName = str(node.getAttribute( "name" ))
            paramType = str(node.getAttribute( "type" ))
            paramArraySize = 1
            if( node.hasAttribute( "arraySize" ) ):
                paramArraySize = int( node.getAttribute( "arraySize" ) )
            paramNiceName = spaceOutCamelCase( paramName )
            paramName = "airs_%s" % paramName
            createControl( shaderNodeName, paramName, paramNiceName, paramType, paramArraySize, parentCtrl )
    print( "" )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param paramType
# @param paramArraySize
# @param parentCtrl
#
def createControl( shaderNodeName, paramName, paramNiceName, paramType, paramArraySize, parentCtrl ):
    if "float" == paramType and 1 == paramArraySize:
        createFloatControl( shaderNodeName, paramName, paramNiceName, parentCtrl )
    elif "float" == paramType and 2 == paramArraySize:
        createFloat2Control( shaderNodeName, paramName, paramNiceName, parentCtrl )
    elif "float" == paramType and 3 == paramArraySize:
        createFloat3Control( shaderNodeName, paramName, paramNiceName, parentCtrl )
    elif "float" == paramType and 16 == paramArraySize:
        createMatrix4x4Control( shaderNodeName, paramName, paramNiceName, parentCtrl )
    elif "string" == paramType:
        createStringControl( shaderNodeName, paramName, paramNiceName, parentCtrl )
    elif "color" == paramType:
        createColorControl( shaderNodeName, paramName, paramNiceName, parentCtrl )
    elif "point" == paramType:
        createFloat3Control( shaderNodeName, paramName, paramNiceName, parentCtrl )
    else:
        print( "[AS4m] : (*** WARNING ***) Unsupported parameter type: type=%s, arraySize=%d (shaders.createControl)" % (paramType, paramArraySize ) )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param parentCtrl
#
def createFloatControl( shaderNodeName, paramName, paramNiceName, parentCtrl ):
    attr = "%s.%s" % (shaderNodeName, paramName)
    cmds.attrFieldSliderGrp( parent=parentCtrl, label=paramNiceName, attribute=attr )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param parentCtrl
#
def createFloat2Control( shaderNodeName, paramName, paramNiceName, parentCtrl ):
    attr = "%s.%s" % (shaderNodeName, paramName)
    cmds.attrFieldGrp( parent=parentCtrl, label=paramNiceName, attribute=attr )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param parentCtrl
#
def createFloat3Control( shaderNodeName, paramName, paramNiceName, parentCtrl ):
    attr = "%s.%s" % (shaderNodeName, paramName)
    cmds.attrFieldGrp( parent=parentCtrl, label=paramNiceName, attribute=attr )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param parentCtrl
#
def filePathBrowseCommand( attr ):
    cmd = 'AEairstreamSurfaceShaderTextureBrowse( "%s" )' % attr
    mel.eval( cmd )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param parentCtrl
#
def createStringControl( shaderNodeName, paramName, paramNiceName, parentCtrl ):
    attr = "%s.%s" % (shaderNodeName, paramName)
    # If the paramName is found in the File Path Attribute array, then create
    # a file path control.
    if not airstream.globals.isFilePathAttrIgnore( paramName ):
        parentLayout = cmds.rowLayout( parent=parentCtrl, nc=3, cw3=[145, 207, 29] )
        cmds.text( label=paramNiceName )
        ctrl = cmds.textField( width=207 )
        cmds.connectControl( ctrl, attr )
        # Browse button
        cmds.rowLayout( nc=1, width=29, columnAttach=[1, "left", 3] )
        cmds.iconTextButton( i1="navButtonBrowse.xpm", align="left", w=29, h=20, command=('airstream.shaders.filePathBrowseCommand("%s")' % attr) )
        cmds.setParent( ".." )
        cmds.setParent( ".." )
    # Create a string control otherwise
    else:
        cmds.rowLayout( parent=parentCtrl, nc=2, cw=[2,207] )
        cmds.text( label=paramNiceName )
        ctrl = cmds.textField( width=207 )
        cmds.connectControl( ctrl, attr )
        cmds.setParent( ".." )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param parentCtrl
#
def createColorControl( shaderNodeName, paramName, paramNiceName, parentCtrl ):
    attr = "%s.%s" % (shaderNodeName, paramName)
    cmds.attrColorSliderGrp( parent=parentCtrl, label=paramNiceName, attribute=attr )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param parentCtrl
#
def createMatrix4x4Control( shaderNodeName, paramName, paramNiceName, parentCtrl ):
    #print( "[PYDBG] : createMatrix4x4Control paramName=%s" % paramName )

    attr = "%s.%s" % (shaderNodeName, paramName)
    matData = cmds.getAttr( attr )
    r0 = matData[0:4]
    r1 = matData[4:8]
    r2 = matData[8:12]
    r3 = matData[12:16]
    ctrl = cmds.columnLayout( adj=False, cw=210 )
    callback = "airstream.shaders.updateMatrix44CallBack('%s', '%s', '%s')" % (shaderNodeName, paramName, ctrl)
    mayaVersion = cmds.about( v=True ).split(" ")[0]
    fVer = float(mayaVersion)
    if fVer < 2008:
        cmds.floatFieldGrp( nf=4, value=r0, cc=callback, cw5=[145, 53, 52, 52, 53], label=paramNiceName )
        cmds.floatFieldGrp( nf=4, value=r1, cc=callback, cw5=[145, 53, 52, 52, 53] )
        cmds.floatFieldGrp( nf=4, value=r2, cc=callback, cw5=[145, 53, 52, 52, 53] )
        cmds.floatFieldGrp( nf=4, value=r3, cc=callback, cw5=[145, 53, 52, 52, 53] )
    else:
        cmds.setUITemplate( "attributeEditorMatrixTemplate", pst=True )
        cmds.floatFieldGrp( nf=4, value=r0, cc=callback, cw5=[145, 53, 52, 52, 53], label=paramNiceName )
        cmds.floatFieldGrp( nf=4, value=r1, cc=callback, cw5=[145, 53, 52, 52, 53] )
        cmds.floatFieldGrp( nf=4, value=r2, cc=callback, cw5=[145, 53, 52, 52, 53] )
        cmds.floatFieldGrp( nf=4, value=r3, cc=callback, cw5=[145, 53, 52, 52, 53] )
        cmds.setUITemplate( ppt=True )
        cmds.setParent( ".." )
    pass

##
# @param shaderNodeName
# @param paramName
# @param containerCtrl
#
def updateMatrix44CallBack( shaderNodeName, paramName, containerCtrl ):
    #print( "[PYDBG] : updateMatrix44CallBack node=%s, paramName=%s" % (shaderNodeName, paramName) )

    grps = cmds.columnLayout( containerCtrl, q=True, ca=True )
    r0 = cmds.floatFieldGrp( grps[0], q=True, value=True )
    r1 = cmds.floatFieldGrp( grps[1], q=True, value=True )
    r2 = cmds.floatFieldGrp( grps[2], q=True, value=True )
    r3 = cmds.floatFieldGrp( grps[3], q=True, value=True )
    matData = r0 + r1 + r2 + r3
    matData = map(str, matData)
    matData = " ".join(matData)
    attr = "%s.%s" % (shaderNodeName, paramName)
    cmd = 'setAttr -type "matrix" %s %s' % (attr, matData)
    print( cmd )
    mel.eval( cmd )
    pass

##
# @param shaderNodeName name of airstreamSurfaceShader node
#
def shaderNameFromXML( shaderNodeName ):
    result = None;
    xml = fetchShaderXml( shaderNodeName )
    #cmds.getAttr( "%s.shaderParams" % shaderNodeName )
    if xml is not None:
        events = pulldom.parseString( xml )
        for (event, node) in events:
            if event == pulldom.START_ELEMENT and node.tagName == "shader":
                events.expandNode( node )
                result = str(node.getAttribute( "name" ) )
                break
    return result
    pass

## Creates attributes to match shader parameters on specified @b node.
# @param shaderNodeName name of airstreamSurfaceShader node
#
# Shader parameter information is contained in XML format in the @a shaderParams attribute.
#
def createParamAttributes( shaderNodeName ):
    #print( "[PYDBG] : Loading shader parameters into %s" % shaderNodeName )
    return

    xml = None
    mayaVersion = cmds.about( v=True ).split(" ")[0]
    fVer = float(mayaVersion)
    if fVer < 2008 and cmds.about( linux=True ):
        xml = mel.eval( "getAttr %s.shaderParams" % shaderNodeName )
    else:
        xml = fetchShaderXml( shaderNodeName )
        #cmds.getAttr( "%s.shaderParams" % shaderNodeName, asString=True )

    if xml is None:
        OpenMaya.MGlobal.displayError( "Shader parameter data missing in %s" % shaderNodeName )
        return
    if "" == xml:
        OpenMaya.MGlobal.displayError( "Shader parameter is empty in %s" % shaderNodeName )
        return
    events = pulldom.parseString( xml )
    for (event, node) in events:
        if event == pulldom.START_ELEMENT and node.tagName == "param":
            events.expandNode( node )
            paramName = str(node.getAttribute( "name" ))
            paramType = str(node.getAttribute( "type" ))
            paramArraySize = 1
            if( node.hasAttribute( "arraySize" ) ):
                paramArraySize = int( node.getAttribute( "arraySize" ) )
            paramData = getParamData( node )
            paramNiceName = paramName
            paramName = "airs_%s" % paramName
            createParam( shaderNodeName, paramName, paramNiceName, paramType, paramData, paramArraySize )
    print( "" )
    pass

##
# @param node
#
def getParamData( node ):
    result = None
    for childNode in node.childNodes:
        if childNode.nodeType == node.TEXT_NODE:
            result = childNode.data
    return result
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param paramType
# @param paramData
# @param paramArraySize
#
def createParam( shaderNodeName, paramName, paramNiceName, paramType, paramData, paramArraySize ):
    if "float" == paramType and 1 == paramArraySize:
        createFloatParam( shaderNodeName, paramName, paramNiceName, paramData )
    elif "float" == paramType and 2 == paramArraySize:
        createFloat2Param( shaderNodeName, paramName, paramNiceName, paramData )
    elif "float" == paramType and 3 == paramArraySize:
        createPointParam( shaderNodeName, paramName, paramNiceName, paramData )
    elif "float" == paramType and 16 == paramArraySize:
        createMatrix4x4Param( shaderNodeName, paramName, paramNiceName, paramData )
    elif "string" == paramType:
        createStringParam( shaderNodeName, paramName, paramNiceName, paramData )
    elif "color" == paramType:
        createColorParam( shaderNodeName, paramName, paramNiceName, paramData )
    elif "point" == paramType:
        createPointParam( shaderNodeName, paramName, paramNiceName, paramData )
    else:
        print( "[AS4M] : (*** WARNING ***) Unsupported parameter type: type=%s, arraySize=%d (shaders.createParam)" % (paramType, paramArraySize ) )
    pass

## Deletes attributes for shader parameters on specified node.
# @param shaderNodeName name of airstreamSurfaceShader node
#
# Looks through all the attributes, anything that begins with "airs_" and exists
# is removed. The existeance check to to make sure that we don't try to remove children
# attributes after the parent has been removed.
#
def deleteParams( shaderNodeName ):
    #print( "[PYDBG] : Deleting shader parameters for %s" % shaderNodeName )

    attributesList = cmds.listAttr( shaderNodeName )
    for attr in attributesList:
        if attr.startswith( "airs_" ) and cmds.attributeQuery( attr, node=shaderNodeName, exists=True ):
            print( "[PYNFO]: Removing %s" % attr )
            cmds.deleteAttr( shaderNodeName, attribute=attr )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param paramData
#
def createFloatParam( shaderNodeName, paramName, paramNiceName, paramData ):
    ## NOTE: Maya 8.5 does not have a nn (aka niceName) so we use the shortName instead.
    mayaVersion = cmds.about( v=True ).split(" ")[0]
    fVer = float(mayaVersion)
    uName = paramName.upper();
    if uName.find("SAMPLE") != -1 :
      if fVer < 2008:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, sn=paramNiceName, at="double", dv=float(paramData), smn=1.0, smx=32.0 )
      else:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, nn=paramNiceName, at="double", dv=float(paramData), smn=1.0, smx=32.0 )
      pass
    else:
      if fVer < 2008:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, sn=paramNiceName, at="double", dv=float(paramData), smn=0.0, smx=1.0 )
      else:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, nn=paramNiceName, at="double", dv=float(paramData), smn=0.0, smx=1.0 )
      pass
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param paramData
#
def createFloat2Param( shaderNodeName, paramName, paramNiceName, paramData ):
    pointData = paramData
    if len( pointData.split() ) != 2:
        pointData = "0 0"
    xval=float(pointData.split()[0])
    yval=float(pointData.split()[1])
    #print( "[PYDBG] : createFloat2Param paramName=%s" % paramName )

    ## NOTE: Maya 8.5 does not have a nn (aka niceName) so we use the shortName instead.
    mayaVersion = cmds.about( v=True ).split(" ")[0]
    fVer = float(mayaVersion)
    if fVer < 2008:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, sn=paramNiceName, at="float2" )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sX"%paramName), sn=("%sX"%paramNiceName), at="float", dv=xval, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sY"%paramName), sn=("%sY"%paramNiceName), at="float", dv=yval, parent=paramName )
    else:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, nn=paramNiceName, at="float2" )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sX"%paramName), nn=("%sX"%paramNiceName), at="float", dv=xval, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sY"%paramName), nn=("%sY"%paramNiceName), at="float", dv=yval, parent=paramName )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param paramData
#
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param paramData
#
def createStringParam( shaderNodeName, paramName, paramNiceName, paramData ):
    ## NOTE: Maya 8.5 does not have a nn (aka niceName) so we use the shortName instead.
    mayaVersion = cmds.about( v=True ).split(" ")[0]
    fVer = float(mayaVersion)
    if fVer < 2008:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, sn=paramNiceName, dt="string" )
    else:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, nn=paramNiceName, dt="string" )
    attr = "%s.%s" % (shaderNodeName, paramName)
    if paramData is not None:
        cmds.setAttr( attr, str( paramData ), type="string" )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param paramData
#
def createColorParam( shaderNodeName, paramName, paramNiceName, paramData ):
    colorData = paramData
    if len( colorData.split() ) != 3:
        colorData = "1 1 1"
    red=float(colorData.split()[0])
    gre=float(colorData.split()[1])
    blu=float(colorData.split()[2])
    ## NOTE: Maya 8.5 does not have a nn (aka niceName) so we use the shortName instead.
    mayaVersion = cmds.about( v=True ).split(" ")[0]
    fVer = float(mayaVersion)
    if fVer < 2008:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, sn=paramNiceName, uac=True, at="float3" )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sR"%paramName), sn=("%sR"%paramNiceName), at="float", dv=red, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sG"%paramName), sn=("%sG"%paramNiceName), at="float", dv=gre, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sB"%paramName), sn=("%sB"%paramNiceName), at="float", dv=blu, parent=paramName )
    else:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, nn=paramNiceName, uac=True, at="float3" )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sR"%paramName), nn=("%sR"%paramNiceName), at="float", dv=red, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sG"%paramName), nn=("%sG"%paramNiceName), at="float", dv=gre, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sB"%paramName), nn=("%sB"%paramNiceName), at="float", dv=blu, parent=paramName )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param paramData
#
def createPointParam( shaderNodeName, paramName, paramNiceName, paramData ):
    pointData = paramData
    if len( pointData.split() ) != 3:
        pointData = "0 0 0"
    xval=float(pointData.split()[0])
    yval=float(pointData.split()[1])
    zval=float(pointData.split()[2])
    ## NOTE: Maya 8.5 does not have a nn (aka niceName) so we use the shortName instead.
    mayaVersion = cmds.about( v=True ).split(" ")[0]
    fVer = float(mayaVersion)
    if fVer < 2008:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, sn=paramNiceName, uac=True, at="float3" )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sX"%paramName), sn=("%sX"%paramNiceName), at="float", dv=xval, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sY"%paramName), sn=("%sY"%paramNiceName), at="float", dv=yval, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sZ"%paramName), sn=("%sZ"%paramNiceName), at="float", dv=zval, parent=paramName )
    else:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, nn=paramNiceName, uac=True, at="float3" )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sX"%paramName), nn=("%sX"%paramNiceName), at="float", dv=xval, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sY"%paramName), nn=("%sY"%paramNiceName), at="float", dv=yval, parent=paramName )
        cmds.addAttr( shaderNodeName, h=True, ln=("%sZ"%paramName), nn=("%sZ"%paramNiceName), at="float", dv=zval, parent=paramName )
    pass

##
# @param shaderNodeName
# @param paramName
# @param paramNiceName
# @param paramData
#
def createMatrix4x4Param( shaderNodeName, paramName, paramNiceName, paramData ):
    #print( "[PYDBG] : createMatrix4x4Param paramName=%s" % paramName )

    ## NOTE: Maya 8.5 does not have a nn (aka niceName) so we use the shortName instead.
    mayaVersion = cmds.about( v=True ).split(" ")[0]
    fVer = float(mayaVersion)
    if fVer < 2008:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, sn=paramNiceName, dt="matrix" )
    else:
        cmds.addAttr( shaderNodeName, h=True, ln=paramName, nn=paramNiceName, dt="matrix" )
    matData = paramData
    if len( matData.split() ) != 16:
        matData = "1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1"
    attr = "%s.%s" % (shaderNodeName, paramName)
    cmd = 'setAttr -type "matrix" %s %s' % (attr, matData)
    mel.eval( cmd )
    pass

